<?php

/*  Alternate WebshopEngine
(C) 2009-2016 by Alternate Tools
-----------------------------------------------------

File:         class_order.php
Description:  Class cOrder definition,
              handling of the webshop order parameters

*/

include "class_address.php";
include "class_basket.php";
include "class_article.php";
include "class_mail.php";

if (!defined("CLASS_ORDER"))
{
    define ("CLASS_ORDER", "DEFINED");

    class cOrder extends cAddress
    {
        var $m_ordId;
        var $m_ordDate;
        var $m_ordFolder;
        var $m_ordFolderDeleted;
        var $m_ordDone;
        var $m_ordDoneDate;
        var $m_ordShippingType;
        var $m_ordShippingCost;
        var $m_ordCustFieldCount;
        var $m_ordVersion;
        var $m_Splitter;
        var $m_CurVersion;

        var $m_ordBasket;

        // PHP 5
        function __construct()
        {
            $this->InitOrder();
            $this->InitAddr();
        }

        // PHP 4
        function cOrder()
        {
            $this->__construct();
        }

        function InitOrder()
        {
            $this->m_ordId = "0";
            $this->m_ordDate = "";
            $this->m_ordFolder = "";
            $this->m_ordFolderDeleted = "";
            $this->m_ordDone = "0";
            $this->m_ordDoneDate = "";
            $this->m_ordShippingType = "";
            $this->m_ordShippingCost = "0";
            $this->m_ordCustFieldCount = 5;
            $this->m_CurVersion = "1.100";
            $this->m_ordVersion = $this->m_CurVersion;

            $this->m_ordBasket = new cBasket;
            $this->m_ordBasket->InitBasket();
            $this->SetCustFieldCount($this->m_ordCustFieldCount);

            $this->m_Splitter = '.|-.';
        }

        // Getter and setter
        function GetId()
        {
            return $this->m_ordId;
        }

        function GetDate()
        {
            return $this->m_ordDate;
        }

        function IsDone()
        {
            return ($this->m_ordDone == "1");
        }
        
        function SetDone($done)
        {
            if ($done == true)
            {
                $this->m_ordDone = "1";
                $this->m_ordDoneDate = date("Y-m-d H:i:s");
            }
            else
            {
                $this->m_ordDone = "0";
                $this->m_ordDoneDate = date("");
            }
        }

        function SetShippingType($shipType)
        {
            $this->m_ordShippingType = $shipType;
        }
        function GetShippingType()
        {
            return $this->m_ordShippingType;
        }

        function SetShippingCost($shipCost)
        {
            $this->m_ordShippingCost = $shipCost;
        }
        function GetShippingCost()
        {
            return $this->m_ordShippingCost;
        }

        function SetCustFieldCount($count)
        {
            $this->m_ordCustFieldCount = $count;
            $this->m_ordBasket->SetCustFieldCount($this->m_ordCustFieldCount);
        }

        function SetFolder($folder)
        {
            $this->m_ordFolder = $folder;
        }
        
        function SetFolderDeleted($folder)
        {
            $this->m_ordFolderDeleted = $folder;
        }

        function GetVersion()
        {
            return $this->m_ordVersion;
        }

        // Main methods
        
        function CreateOrder()
        {
            $this->InitOrder();
            $this->m_ordId = uniqid("ord", FALSE);
            $this->m_ordDate = date("Y-m-d H:i:s");
        }

        function DeleteOrderFinal()  // Move to trash
        {
            if (copy($this->m_ordFolder . $this->m_ordId . ".ord", $this->m_ordFolderDeleted . $this->m_ordId . ".ord") == false)
                return false;
            return unlink($this->m_ordFolder . $this->m_ordId . ".ord");
        }
        function DeleteOrderFinalX()  // Unlink -> not useful!!
        {
            return unlink($this->m_ordFolder . $this->m_ordId . ".ord");
        }

        function DeleteOrderFinalFromId($ordId)  // Move to trash
        {
            if (copy($this->m_ordFolder . $ordId . ".ord", $this->m_ordFolderDeleted . $ordId . ".ord") == false)
                return false;
            return unlink($this->m_ordFolder . $ordId . ".ord");
        }
        function DeleteOrderFinalFromIdX($ordId)  // Unlink -> not useful!!
        {
            return unlink($this->m_ordFolder . $ordId . ".ord");
        }

        function LoadOrderFromId($ordId)
        {
            $this->m_ordId = $ordId;
            return $this->LoadOrder();
        }

        function LoadOrder()
        {
            $fileName = $this->m_ordFolder . $this->m_ordId  . ".ord";
            // echo  $fileName;

            if ($this->m_ordId == "0" || $this->m_ordFolder == "")
                return false;

            $cStorage = new cStorageFile;
            if ($cStorage->LoadStorageFromFile($fileName) == false)
                return false;

            if ($cStorage->GetData("Id") == "")
                return false;

            $this->m_ordVersion = $cStorage->GetVersion();
            $this->m_ordId = $cStorage->GetData("Id");
            $this->m_ordDate = $cStorage->GetData("Date");
            $this->m_ordDone = $cStorage->GetData("Done");
            $this->m_ordDoneDate = $cStorage->GetData("DoneDate");
            $this->m_ordShippingType = $cStorage->GetData("ShipType");
            $this->m_ordShippingCost = $cStorage->GetData("ShipCost");
            // Address
            $this->SetFirstName($cStorage->GetData("FirstName"));
            $this->SetLastName($cStorage->GetData("LastName"));
            $this->SetStreet($cStorage->GetData("Street"));
            $this->SetStreetNo($cStorage->GetData("StreetNo"));
            $this->SetCity($cStorage->GetData("City"));
            $this->SetZipCode($cStorage->GetData("ZipCode"));
            $this->SetLand($cStorage->GetData("Land"));
            $this->SetTelephoneNo($cStorage->GetData("TelephoneNo"));
            $this->SetEmail($cStorage->GetData("Email"));
            // Basket
            $this->m_ordBasket->m_aArtIds = $this->OrdExlpode($cStorage->GetData("BasketIds"));
            $this->m_ordBasket->m_aCounts = $this->OrdExlpode($cStorage->GetData("BasketCounts"));
            $this->m_ordBasket->m_aPrices = $this->OrdExlpode($cStorage->GetData("BasketPrices"));
            $this->m_ordBasket->m_aNames = $this->OrdExlpode($cStorage->GetData("BasketNames"));
            $this->m_ordBasket->m_aAddToPrices = $this->OrdExlpode($cStorage->GetData("BasketAddToPrices"));
            // Custom fields
            for ($i = 1; $i <= $this->m_ordCustFieldCount; $i ++)       // ??? PHP 4.4??
            {
                $this->m_ordBasket->m_aCustNames[$i - 1] = $this->OrdExlpode($cStorage->GetData("BasketCNames" . $i));
                $this->m_ordBasket->m_aCustFields[$i - 1] = $this->OrdExlpode($cStorage->GetData("BasketCFields" . $i));
            }

            return true;
        }

        function SaveOrder()
        {
            $newRow = "\r\n";
            $fileName = $this->m_ordFolder . $this->m_ordId . ".ord";
            // echo $fileName;

            if ($this->m_ordId == "0" || $this->m_ordFolder == "")
                return false;

            $cStorage = new cStorageFile;
            $cStorage->SetType("order");
            $cStorage->SetVersion($this->m_CurVersion);
            $cStorage->AddData("Id", $this->m_ordId);
            $cStorage->AddData("Date", $this->m_ordDate);
            $cStorage->AddData("Done", $this->m_ordDone);
            $cStorage->AddData("DoneDate", $this->m_ordDoneDate);
            $cStorage->AddData("ShipType", $this->m_ordShippingType);
            $cStorage->AddData("ShipCost", $this->m_ordShippingCost, false);
            // Address
            $cStorage->AddData("FirstName", $this->GetFirstName());
            $cStorage->AddData("LastName", $this->GetLastName());
            $cStorage->AddData("Street", $this->GetStreet());
            $cStorage->AddData("StreetNo", $this->GetStreetNo());
            $cStorage->AddData("City", $this->GetCity());
            $cStorage->AddData("ZipCode", $this->GetZipCode());
            $cStorage->AddData("Land", $this->GetLand());
            $cStorage->AddData("TelephoneNo", $this->GetTelephoneNo());
            $cStorage->AddData("Email", $this->GetEmail());
            // Basket
            $cStorage->AddData("BasketIds", implode($this->m_Splitter , $this->m_ordBasket->m_aArtIds));
            $cStorage->AddData("BasketCounts", implode($this->m_Splitter , $this->m_ordBasket->m_aCounts));
            $cStorage->AddData("BasketPrices", implode($this->m_Splitter , $this->m_ordBasket->m_aPrices));
            $cStorage->AddData("BasketNames", implode($this->m_Splitter , $this->m_ordBasket->m_aNames));
            $cStorage->AddData("BasketAddToPrices", implode($this->m_Splitter , $this->m_ordBasket->m_aAddToPrices));
            // Custom fields
            for ($i = 1; $i <= $this->m_ordCustFieldCount; $i ++)
            {
                $cStorage->AddData("BasketCNames" . $i, implode($this->m_Splitter , $this->m_ordBasket->m_aCustNames[$i - 1]));
                $cStorage->AddData("BasketCFields" . $i, implode($this->m_Splitter , $this->m_ordBasket->m_aCustFields[$i - 1]));
            }

            return $cStorage->SaveStorageToFile($fileName);
        }
        
        function SendOrder($emailDest, $emailSrc, $aTranslations, $artFolder, $mailFolder)
        {
            $newRow = "\r\n";
            $artArticle = new cArticle;
            $artArticle->SetFolder($artFolder);
            $artArticle->SetCustFieldCount($this->m_ordCustFieldCount);

            $Title = $aTranslations['title'] . ': '. $this->GetId();
            $Message = $aTranslations['title'] . ': '. $this->GetId() . $newRow;
            // Address
            $Message = $Message . ($aTranslations['address'] . ':' . $newRow);
            $Message = $Message . ($this->GetFirstName() . ' ' . $this->GetLastname() . $newRow);
            $Message = $Message . ($this->GetStreet() . ' ' . $this->GetStreetNo() . $newRow);
            $Message = $Message . ($this->GetZipCode() . ' ' . $this->GetCity() . $newRow);
            $Message = $Message . ($this->GetLand() . $newRow);
            $Message = $Message . ($aTranslations['email'] . ': ' . $this->GetEmail() . $newRow);
            $Message = $Message . ($aTranslations['phoneno'] . ': ' . $this->GetTelephoneNo() . $newRow);
            $Message = $Message . ($aTranslations['shiptype'] . ': ' . $this->GetShippingType() . $newRow);
            $Message = $Message . ($aTranslations['articles'] . ': ' . $newRow);
            // Articles
            for ($i = 0; $i < count($this->m_ordBasket->m_aArtIds); $i ++)
            {
                $custMessage = "";
                $artArticle->LoadArticleFromId($this->m_ordBasket->m_aArtIds[$i]);
                $Message = $Message . ($aTranslations['id'] . ': ' . $this->m_ordBasket->m_aArtIds[$i] . $newRow);
                $Message = $Message . ($aTranslations['name'] . ': ' . $artArticle->GetName() . $newRow);
                $Message = $Message . ($aTranslations['count'] . ': ' . $this->m_ordBasket->m_aCounts[$i] . $newRow);
                // Custom fields
                for ($v = 1; $v <= $this->m_ordCustFieldCount; $v ++)
                {
                     $aCNames = $this->m_ordBasket->GetCustNames($v);
                     $aCFields = $this->m_ordBasket->GetCustFields($v);
                     if ($aCNames[$i] != "")
                         $custMessage = $custMessage . ($aCNames[$i] . '=' . $aCFields[$i] . $newRow);
                }
                // Check, if custom fields are used
                if ($custMessage != "")
                {
                    $Message = $Message . ($aTranslations['userdef'] . ': ' . $newRow);
                    $Message = $Message . ($custMessage . $newRow);
                }
            }

            $Message = $Message . ($aTranslations['hint'] . $newRow);
            // Send email
            $cEmail = new cMail;
            return $cEmail->SendMailByParams($emailDest, $Title, $Message, $emailSrc, $mailFolder);
        }

        function SendCustomerOrder($emailDest, $emailSrc, $aTranslations, $artFolder, $footer, $currency, $mailFolder)
        {
            $newRow = "\r\n";
            $artArticle = new cArticle;
            $artArticle->SetFolder($artFolder);
            $artArticle->SetCustFieldCount($this->m_ordCustFieldCount);
            $artPrice = 0;

            $Title = $aTranslations['title'] . ': '. $this->GetId();
            $Message = $aTranslations['articles'] . ': ' . $newRow . $newRow;
            // Articles
            for ($i = 0; $i < count($this->m_ordBasket->m_aArtIds); $i ++)
            {
                $custMessage = "";
                $artArticle->LoadArticleFromId($this->m_ordBasket->m_aArtIds[$i]);
                $Message = $Message . ('#' . ($i + 1) . $newRow);
                $Message = $Message . ($aTranslations['name'] . ': ' . $artArticle->GetName() . $newRow);
                $Message = $Message . ($aTranslations['count'] . ': ' . $this->m_ordBasket->m_aCounts[$i] . $newRow);
                $Message = $Message . ($aTranslations['price'] . ': ' . $this->FormatPrice($artArticle->GetPrice()) . ' ' . $currency . $newRow);
                // Custom fields
                for ($v = 1; $v <= $this->m_ordCustFieldCount; $v ++)
                {
                     $aCNames = $this->m_ordBasket->GetCustNames($v);
                     $aCFields = $this->m_ordBasket->GetCustFields($v);
                     if (isset($aCNames[$i]) == true &&  $aCNames[$i] != "")
                         $custMessage = $custMessage . ($aCNames[$i] . '=' . $aCFields[$i] . $newRow);
                }
                // Check, if custom fields are used
                if ($custMessage != "")
                {
                    $Message = $Message . ($aTranslations['userdef'] . ': ' . $newRow);
                    $Message = $Message . ($custMessage . $newRow);
                }
                $artPrice = $artPrice + ($artArticle->GetPrice() * $this->m_ordBasket->m_aCounts[$i]);
            }
            $Message = $Message . ($newRow . $aTranslations['shiptype'] . ': ' . $this->GetShippingType() . ' (+ ' . $this->m_ordShippingCost . ' ' . $currency . ')' . $newRow);
            $Message = $Message . ($newRow . $aTranslations['totalprice'] . ': ' . $this->FormatPrice($artPrice + $this->m_ordShippingCost) . ' ' . $currency . $newRow);
            $Message = $Message . ($newRow . $footer . $newRow);

            // Send email
            $cEmail = new cMail;
            return $cEmail->SendMailByParams($emailDest, $Title, $Message, $emailSrc, $mailFolder);
        }

        function OrdExlpode($strString)
        {
            $outArray = array();
            if ($strString != "")
            {
                if (strstr($strString, $this->m_Splitter) == false)
                {
                    $outArray[] = $strString;
                }
                else
                {
                    $outArray = explode($this->m_Splitter, $strString);
                }
            }
            return $outArray;
        }
        
        function FormatPrice($priceString)
        {
            return sprintf("%01.2f", $priceString);
        }

        // Basket access methods
        
        function GetArticleIds()
        {
            return $this->m_ordBasket->GetArticleIds();
        }
        function GetArticleCounts()
        {
            return $this->m_ordBasket->GetCounts();
        }
        function GetArticlePrices()
        {
            return $this->m_ordBasket->GetPrices();
        }
        function SetArticlePrices($prices)
        {
            $this->m_ordBasket->SetPrices($prices);
        }
        function GetArticleNames()
        {
            return $this->m_ordBasket->GetNames();
        }
        function SetArticleNames($names)
        {
            $this->m_ordBasket->SetNames($names);
        }
        function GetArticleAddToPrices()
        {
            return $this->m_ordBasket->GetAddToPrices();
        }
        function GetTotalArticleCount()
        {
            return $this->m_ordBasket->GetTotalArticleCount();
        }
        function GetArticlePrice($index)
        {
            return $this->m_ordBasket->GetArticlePrice($index);
        }
        function GetTotalArticlePrice()
        {
            return $this->m_ordBasket->GetTotalArticlePrice();
        }
        function GetTotalOrderPrice()
        {
            return $this->GetTotalArticlePrice() + $this->m_ordShippingCost;
        }
        function GetArticleCustNames($index)
        {
            return $this->m_ordBasket->GetArticleCustNames($index);
        }
        function GetArticleCustFields($index)
        {
            return $this->m_ordBasket->GetArticleCustFields($index);
        }

    }
}